import sys
import os
import random
import subprocess
from PyQt5.QtWidgets import (QApplication, QMainWindow, QVBoxLayout, QWidget,
                             QLineEdit, QTabWidget, QPushButton, QShortcut, QColorDialog, QDialog, QCheckBox, QLabel, QHBoxLayout, QMessageBox, QTextEdit)
from PyQt5.QtGui import QKeySequence, QColor, QPixmap
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEnginePage, QWebEngineSettings
from PyQt5.QtCore import QUrl, QTimer, Qt, QSettings
import urllib.request

REQUIRED_LIBRARIES = ["PyQt5", "PyQtWebEngine"]

def install_libraries():
    """Installs required libraries if not already available."""
    for lib in REQUIRED_LIBRARIES:
        try:
            __import__(lib)
        except ImportError:
            subprocess.run([sys.executable, "-m", "pip", "install", lib], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

install_libraries()

def ensure_assets():
    """Ensures necessary assets are available."""
    assets_dir = os.path.join(os.path.dirname(__file__), "assets")
    if not os.path.exists(assets_dir):
        os.makedirs(assets_dir)

    crack_image_url = "https://static.vecteezy.com/system/resources/thumbnails/018/972/639/small_2x/abstract-broken-glass-texture-with-particles-png.png"
    crack_image_path = os.path.join(assets_dir, "crack_image.png")

    if not os.path.exists(crack_image_path):
        try:
            req = urllib.request.Request(crack_image_url, headers={'User-Agent': 'Mozilla/5.0'})
            with urllib.request.urlopen(req) as response, open(crack_image_path, 'wb') as out_file:
                out_file.write(response.read())
        except Exception as e:
            print(f"Error downloading crack image: {e}")

ensure_assets()

class SettingsDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Settings")
        self.setGeometry(200, 200, 300, 300)

        layout = QVBoxLayout()

        self.color_button = QPushButton("Change Browser Color")
        self.color_button.clicked.connect(self.change_color)
        layout.addWidget(self.color_button)

        self.shake_checkbox = QCheckBox("Enable Shake")
        layout.addWidget(self.shake_checkbox)

        self.crack_checkbox = QCheckBox("Enable Screen Crack")
        layout.addWidget(self.crack_checkbox)

        self.highlight_checkbox = QCheckBox("Enable Random Highlighter")
        layout.addWidget(self.highlight_checkbox)

        self.epok_audio_checkbox = QCheckBox("Enable Epok Audio Quality (Bass Boost)")
        layout.addWidget(self.epok_audio_checkbox)

        self.history_button = QPushButton("View History")
        self.history_button.clicked.connect(self.open_history)
        layout.addWidget(self.history_button)

        self.apply_button = QPushButton("Apply")
        self.apply_button.clicked.connect(self.apply_settings)
        layout.addWidget(self.apply_button)

        self.setLayout(layout)

        self.shake_checkbox.setChecked(self.parent().shake_enabled)
        self.crack_checkbox.setChecked(self.parent().screen_crack_enabled)
        self.highlight_checkbox.setChecked(self.parent().highlighter_enabled)
        self.epok_audio_checkbox.setChecked(self.parent().epok_audio_enabled)

    def change_color(self):
        """Opens a color picker dialog to change the browser background color."""
        color = QColorDialog.getColor()
        if color.isValid():
            self.parent().change_html_color(color)

    def apply_settings(self):
        """Applies the selected settings and closes the dialog."""
        self.parent().shake_enabled = self.shake_checkbox.isChecked()
        self.parent().screen_crack_enabled = self.crack_checkbox.isChecked()
        self.parent().highlighter_enabled = self.highlight_checkbox.isChecked()
        self.parent().epok_audio_enabled = self.epok_audio_checkbox.isChecked()
        self.parent().save_settings()
        self.parent().apply_screen_crack(self.parent().screen_crack_enabled)
        self.parent().apply_highlighter(self.parent().highlighter_enabled)
        self.parent().apply_epok_audio_quality(self.parent().epok_audio_enabled)
        self.accept()

    def open_history(self):
        """Opens the history dialog."""
        self.parent().open_history()

class ExtensionsDialog(QDialog):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Extensions")
        self.setGeometry(200, 200, 400, 300)

        layout = QVBoxLayout()

        self.extension_label = QLabel("Manage Extensions")
        layout.addWidget(self.extension_label)

        self.example_extension_checkbox = QCheckBox("Enable Example Extension")
        layout.addWidget(self.example_extension_checkbox)

        self.apply_button = QPushButton("Apply")
        self.apply_button.clicked.connect(self.apply_extensions)
        layout.addWidget(self.apply_button)

        self.setLayout(layout)

    def apply_extensions(self):
        """Apply selected extensions."""
        if self.example_extension_checkbox.isChecked():
            print("Example Extension Enabled")
        else:
            print("Example Extension Disabled")
        self.accept()

class HistoryDialog(QDialog):
    def __init__(self, history, parent=None):
        super().__init__(parent)
        self.setWindowTitle("History")
        self.setGeometry(200, 200, 400, 300)

        layout = QVBoxLayout()

        self.history_label = QLabel("Browsing History")
        layout.addWidget(self.history_label)

        self.history_text = QTextEdit()
        self.history_text.setReadOnly(True)
        self.history_text.setPlainText("\n".join(history))
        layout.addWidget(self.history_text)

        self.close_button = QPushButton("Close")
        self.close_button.clicked.connect(self.accept)
        layout.addWidget(self.close_button)

        self.setLayout(layout)

class MewingBrowser(QMainWindow):
    def __init__(self):
        super().__init__()

        self.setWindowTitle("Mewing Browser")
        self.setGeometry(100, 100, 1000, 600)

        self.shake_enabled = True
        self.screen_crack_enabled = False
        self.highlighter_enabled = False
        self.epok_audio_enabled = False
        self.history = []

        self.load_settings()

        self.data_folder = os.path.expanduser("~/.mewing_browser")
        if not os.path.exists(self.data_folder):
            os.makedirs(self.data_folder)

        self.shake_timer = QTimer()
        self.shake_timer.timeout.connect(self.shake_content)
        if self.shake_enabled:
            self.shake_timer.start(100)

        self.tabs = QTabWidget()
        self.tabs.setTabPosition(QTabWidget.West)
        self.tabs.setTabsClosable(True)
        self.tabs.tabCloseRequested.connect(self.close_tab)
        self.setCentralWidget(self.tabs)

        self.add_new_tab(QUrl("https://www.google.com"), "Home")

        self.url_bar = QLineEdit()
        self.url_bar.returnPressed.connect(self.navigate_to_url)

        self.new_tab_button = QPushButton("+")
        self.new_tab_button.clicked.connect(self.add_empty_tab)

        top_layout = QHBoxLayout()
        top_layout.addWidget(self.url_bar)
        top_layout.addWidget(self.new_tab_button)

        top_widget = QWidget()
        top_widget.setLayout(top_layout)

        self.setMenuWidget(top_widget)

        self.settings_shortcut = QShortcut(QKeySequence("Ctrl+S"), self)
        self.settings_shortcut.activated.connect(self.open_settings)


        self.dev_mode_shortcut = QShortcut(QKeySequence("Ctrl+D"), self)
        self.dev_mode_shortcut.activated.connect(self.open_dev_mode)

        self.crack_overlay = QLabel(self)
        crack_image_path = os.path.join(os.path.dirname(__file__), "assets", "crack_image.png")
        self.crack_overlay.setPixmap(QPixmap(crack_image_path))
        self.crack_overlay.setScaledContents(True)
        self.crack_overlay.hide()

        self.zoom_label = QLabel("Zoom: 100%", self)
        self.zoom_label.setStyleSheet("background-color: rgba(0, 0, 0, 0.7); color: white; font-size: 16px; padding: 5px;")
        self.zoom_label.move(20, 20)
        self.zoom_label.hide()

        self.show()

    def add_new_tab(self, url, label):
        """Adds a new tab with the specified URL and label."""
        browser = QWebEngineView()
        browser.setUrl(url)
        browser.titleChanged.connect(lambda title: self.update_tab_title(title, self.tabs.indexOf(browser)))
        browser.urlChanged.connect(self.record_history)
        i = self.tabs.addTab(browser, label)
        self.tabs.setCurrentIndex(i)

    def record_history(self, url):
        """Records visited URLs to history."""
        self.history.append(url.toString())

    def add_empty_tab(self):
        """Adds a new empty tab."""
        self.add_new_tab(QUrl("https://www.google.com"), "New Tab")

    def close_tab(self, index):
        """Closes the tab at the specified index."""
        if self.tabs.count() > 1:
            self.tabs.removeTab(index)
        else:
            QMessageBox.warning(self, "Warning", "You cannot close the last tab.")

    def navigate_to_url(self):
        """Navigates to the URL entered in the URL bar."""
        url = self.url_bar.text()
        if not url.startswith("http://") and not url.startswith("https://"):
            url = "https://" + url
        self.current_browser().setUrl(QUrl(url))

    def current_browser(self):
        """Returns the currently active browser tab."""
        return self.tabs.currentWidget()

    def shake_content(self):
        """Applies a shake effect to the current tab's content."""
        if self.shake_enabled and self.isVisible():
            browser = self.current_browser()
            if browser:
                script = """
                var shakeDuration = 0.1;
                var shakeDistance = 5;
                var shakeInterval = 5;

                var totalShakes = 10;
                var count = 0;

                function shake() {
                    var x = Math.random() * shakeDistance * 2 - shakeDistance;
                    var y = Math.random() * shakeDistance * 2 - shakeDistance;
                    document.body.style.transform = 'translate(' + x + 'px, ' + y + 'px)';
                    count++;
                    if (count < totalShakes) {
                        setTimeout(shake, shakeInterval);
                    } else {
                        document.body.style.transform = 'translate(0, 0)';
                    }
                }

                shake();
                """
                browser.page().runJavaScript(script)

    def change_html_color(self, color):
        """Changes the background color of all tabs to the specified color."""
        color_hex = color.name()
        script = f"document.body.style.backgroundColor = '{color_hex}';"
        for i in range(self.tabs.count()):
            browser = self.tabs.widget(i)
            browser.page().runJavaScript(script)

    def enable_screen_crack(self, enable):
        """Toggles the screen crack overlay effect."""
        if enable:
            self.crack_overlay.setGeometry(self.rect())
            self.crack_overlay.show()
        else:
            self.crack_overlay.hide()

    def enable_highlighter(self):
        """Enables a highlighter effect on all tabs."""
        script = "document.body.style.filter = 'brightness(1.5) contrast(3) saturate(2)';"
        for i in range(self.tabs.count()):
            browser = self.tabs.widget(i)
            browser.page().runJavaScript(script)

    def update_tab_title(self, title, index):
        """Updates the title of a tab."""
        self.tabs.setTabText(index, title)

    def apply_screen_crack(self, enable):
        """Applies or removes the screen crack effect."""
        self.enable_screen_crack(enable)

    def apply_highlighter(self, enable):
        """Applies or removes the highlighter effect."""
        if enable:
            self.enable_highlighter()
        else:
            script = "document.body.style.filter = 'none';"
            for i in range(self.tabs.count()):
                browser = self.tabs.widget(i)
                browser.page().runJavaScript(script)

    def apply_epok_audio_quality(self, enable):
        """Applies a bass boost effect to audio and video elements."""
        script = """
        var audioElements = document.querySelectorAll('audio, video');
        audioElements.forEach(function(audio) {
            var context = new (window.AudioContext || window.webkitAudioContext)();
            var source = context.createMediaElementSource(audio);
            var filter = context.createBiquadFilter();
            filter.type = "lowshelf";
            filter.frequency.value = 200;
            filter.gain.value = 15;
            source.connect(filter);
            filter.connect(context.destination);
        });
        """ if enable else """
        var audioElements = document.querySelectorAll('audio, video');
        audioElements.forEach(function(audio) {
            var context = new (window.AudioContext || window.webkitAudioContext)();
            var source = context.createMediaElementSource(audio);
            source.connect(context.destination);
        });
        """
        for i in range(self.tabs.count()):
            browser = self.tabs.widget(i)
            browser.page().runJavaScript(script)

    def open_settings(self):
        """Opens the settings dialog."""
        settings_dialog = SettingsDialog(self)
        settings_dialog.exec_()

    def open_dev_mode(self):
        """Opens developer mode with the browser's dev tools."""
        dev_tools_script = """
        if (!window.__devToolsOpened) {
            const devTools = document.createElement('div');
            devTools.style.position = 'absolute';
            devTools.style.top = '20px';
            devTools.style.left = '20px';
            devTools.style.backgroundColor = 'rgba(0, 0, 0, 0.7)';
            devTools.style.color = 'white';
            devTools.style.padding = '10px';
            devTools.innerHTML = '<h4>Developer Tools</h4><button id="runJsBtn">Run JS</button><button id="inspectBtn">Inspect Elements</button>';
            document.body.appendChild(devTools);
            
            const runJsButton = document.getElementById('runJsBtn');
            const inspectButton = document.getElementById('inspectBtn');
            
            runJsButton.addEventListener('click', () => {
                const script = prompt('Enter JS to run:');
                if (script) {
                    eval(script);
                }
            });
            
            inspectButton.addEventListener('click', () => {
                alert('Inspecting elements...');  // You can add more functionality here
            });
            
            window.__devToolsOpened = true;
        }
        """
        self.current_browser().page().runJavaScript(dev_tools_script)
        
    def save_settings(self):
        """Saves the settings to a file."""
        settings = QSettings("MewingBrowser", "Settings")
        settings.setValue("shake_enabled", self.shake_enabled)
        settings.setValue("screen_crack_enabled", self.screen_crack_enabled)
        settings.setValue("highlighter_enabled", self.highlighter_enabled)
        settings.setValue("epok_audio_enabled", self.epok_audio_enabled)

    def load_settings(self):
        """Loads the settings from a file."""
        settings = QSettings("MewingBrowser", "Settings")
        self.shake_enabled = settings.value("shake_enabled", True, type=bool)
        self.screen_crack_enabled = settings.value("screen_crack_enabled", False, type=bool)
        self.highlighter_enabled = settings.value("highlighter_enabled", False, type=bool)
        self.epok_audio_enabled = settings.value("epok_audio_enabled", False, type=bool)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MewingBrowser()
    sys.exit(app.exec_())
